#!/usr/bin/python

import argparse
import requests
import json
import csv
import base64

HOST = 'https://console.cloudendure.com'

def read_config_csv(input_file):
	result = []
	with open(input_file, mode='r') as infile:
		reader = csv.DictReader(infile)
		for row in reader:
			result.append(row)
	return result

def login(username, password):
	endpoint = '/api/latest/{}'
	session = requests.Session()
	session.headers.update({'Content-type': 'application/json', 'Accept': 'text/plain'})
	
	resp = session.post(url=HOST+endpoint.format('login'),
									data=json.dumps({'username': username,
													'password': password}))
	if resp.status_code != 200 and resp.status_code != 307:
		log.error('Could not login!')
		sys.exit(2)
		
	# check if need to use a different API entry point
	if resp.history:
		endpoint = '/' + '/'.join(resp.url.split('/')[3:-1]) + '/{}'
		resp = session.post(url=HOST+endpoint.format('login'),
						data=json.dumps({'username': username, 'password': password}))
	if resp.status_code != 200:
		raise Exception('login error')
		
	session.headers['X-XSRF-TOKEN'] = session.cookies.get('XSRF-TOKEN')
		
	return session, endpoint


def main(args):
	
	session, endpoint = login(args.user, args.password)
	
	new_credentials = read_config_csv(args.inputfile)
	
	# Get ID for AWS cloud
	aws_cloud_id = None
	resp = session.get(url=HOST+endpoint.format('/clouds'))
	for clouds_item in json.loads(resp.content)['items']:
		if clouds_item['name'] == 'AWS':
			aws_cloud_id = clouds_item['id']
			break

	assert aws_cloud_id

	# Get exisiting Cloud Creentials
	cloud_credentials = {}
	resp = session.get(url=HOST+endpoint.format('/cloudCredentials'))
	for credentials in json.loads(resp.content)['items']:
		cloud_credentials[credentials['id']] = credentials
		
	# Get exisiting Projects
	projects = {}
	project_ids = {}
	resp = session.get(url=HOST+endpoint.format('/projects'))
	for p in json.loads(resp.content)['items']:
		projects[p['name']] = None
		if p['cloudCredentialsIDs']:
			projects[p['name']] = p['cloudCredentialsIDs'][0]	
		
		project_ids[p['name']] = p['id']
		
	# Set new Credentials
	for credentials in new_credentials:
		creds = {
				'publicKey': credentials['publicKey'],
				'privateKey': base64.b64encode(credentials['privateKey'])
		}
		if credentials['project_name'] in projects:
			creds_id = projects[credentials['project_name']]
			# Set existing credentials for a project
			if creds_id:
				old_credentials = cloud_credentials[creds_id]
				creds['cloudId'] = old_credentials['cloud']
				creds['id'] = creds_id			
				resp = session.patch(url=HOST+endpoint.format('cloudCredentials/{}'.format(creds_id)), data=json.dumps(creds))
				if resp.status_code != 200:
					print 'Failed setting credentails for project {}. Error:{} {}'.format(credentials['project_name'],resp.content, resp.status_code)
				else:
					print 'Credentials for project {} were set successfully'.format(credentials['project_name'])
			# Project does not have credentials, setting for the first time
			else:
				creds['cloudId'] = aws_cloud_id
				resp = session.post(url=HOST+endpoint.format('cloudCredentials'), data=json.dumps(creds))
				if resp.status_code != 201:
					print 'Failed setting credentails for project {}. Error:{} {}'.format(credentials['project_name'],resp.content, resp.status_code)
				else:
					cloud_credentials_id = json.loads(resp.content)['id']
					project_id = project_ids[credentials['project_name']]
					data = {'cloudCredentialsIDs': [cloud_credentials_id]}
					resp = session.patch(url=HOST+endpoint.format('projects/{}'.format(project_id)), data=json.dumps(data))				
					if resp.status_code != 200:
						print 'Failed patch project {} with new credentials {} credentails. Error:{} {}'.format(credentials['project_name'],cloud_credentials_id ,resp.content, resp.status_code)
					else:
						print 'Credentials for project {} were set successfully'.format(credentials['project_name'])
		else:
			print 'Project {} was not found'.format(credentials['project_name'])

	return 0


if __name__ == '__main__':
	
	parser = argparse.ArgumentParser()
	parser.add_argument('-u', '--user', required=True, help='User name')
	parser.add_argument('-p', '--password', required=True, help='Password')
	parser.add_argument('-i', '--inputfile', required=True, help='Input CSV file')

	main(parser.parse_args())
